#include "LCDv1.h"
#include "PCF8574.h"
#include "strings.h"

#define LCD_BackLight_BIT          (1<<3)
#define LCD_RS_BIT                 (1<<0)
#define LCD_E_BIT                  (1<<2)
#define LCD_Data_Mask               0xF0
#define LCD_CMD_Mask                0x0E

unsigned char SPI_DAT=0;
void LCD_Write(u8 Dat);
void LCD_Send_Cmd(u8 CMD);
void LCD_WriteData(u8 dat);

void LCD_init(void)
{
        Delay_ms(17);
        LCD_Write(0x30);
        Delay_ms(1);
        LCD_Write(0x30);
        Delay_ms(1);
        LCD_Write(0x30);
        Delay_ms(1);
        LCD_Write(0x20);  //4 bit wide bus
                  
        LCD_Send_Cmd(0x28);           //4bit wide bus 2 rows
        LCD_Send_Cmd(0x10);
        LCD_Send_Cmd(0x01);
        LCD_Send_Cmd(0x0F);

        LCD_Send_Cmd(LCD_CMD_Cursor_Off);
        LCD_Send_Cmd(LCD_CMD_Clear);
        Delay_ms(10);
}

void LCD_Write(u8 Dat)
{         
 SPI_DAT&=(LCD_RS_BIT|LCD_BackLight_BIT);
 SPI_DAT|=(Dat&LCD_Data_Mask)|LCD_E_BIT;
 PCF8574_Write(SPI_DAT);
 Delay_us(5);
 SPI_DAT&=~LCD_E_BIT;
 PCF8574_Write(SPI_DAT);
 Delay_us(5);
}
void LCD_Send_Cmd(u8 CMD)
{
        Delay_ms(5);
        SPI_DAT&=~LCD_RS_BIT;
        LCD_Write((CMD&0xF0));
        LCD_Write(((CMD&0x0F)<<4));
}

void LCD_SetPosition(u8 row, u8 colum)
{
        u8 position = 0x80;
        if (row > 0) { position |= 0x40;}
        LCD_Send_Cmd(position | colum);
}

void LCD_WriteData(u8 dat)
{
        Delay_us(50);
        SPI_DAT|=LCD_RS_BIT;
        LCD_Write((dat&0xF0));
        LCD_Write(((dat&0x0F)<<4));
}

void LCD_BackLight (unsigned char State)
{
  if (State)  SPI_DAT|=LCD_BackLight_BIT;
  else   SPI_DAT&=~LCD_BackLight_BIT;
}

void LCD_PrintChar(char Data)
{
	if (Data=='\n') LCD_Send_Cmd(0x80|0x40);
	   else LCD_WriteData(Data);
}

void LCD_PrintString(char * string)
{
	while (*string) LCD_PrintChar(*string++);
}

void LCD_PrintStringLen(char * string, unsigned char Len)
{
	while (Len--) LCD_PrintChar(*string++);
}

void LCD_PrintHEX_Byte (unsigned char HexValue)
{
	char String[5]="0x";
	ClearString(String+2,3,0);
	ValueToStringHEX_Byte(HexValue, String+2);
	LCD_PrintString(String);
}

void LCD_PrintValue_uChar (unsigned char Value)
{
	char String[4];
	ClearString(String,4,0);
	uCharToStr(Value, String);
	LCD_PrintString(String);
}

void LCD_PrintValue_uLong (unsigned long Value)
{
	char String[11];
	ClearString(String,11,0);
	uLongToStr(Value, String);
	LCD_PrintString(String);
}

void LCD_PrintValue_sChar (signed char Value)
{
	char String[5];
	ClearString(String,5,0);
	sCharToStr(Value, String);
	LCD_PrintString(String);
}

void LCD_PrintValue_sLong (signed long Value)
{
	char String[12];
	ClearString(String,12,0);
	sLongToStr(Value, String);
	LCD_PrintString(String);
}